;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  hla.mac: simple Linux/X graphics demo                         ;;
;;                                                                ;;
;;           X code based on examples by Frank Kotler             ;;
;;                                                                ;;
;;           convert any HLA to asm                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        ; nasm -O99 -f bin -o hla hla.asm
        %include "mac.inc"

;===========================================================================
        seg 32
        orig equ $08048000
        code_addr equ orig
        code_offset equ 0
        section .text vstart=code_addr

;--------------------------- ELF header -----------------------------------
        dc.l $464c457f,$00010101,0,0,$00030002,1,main,$34,0,0,$00200034,2,0
        dc.l 1,code_offset,code_addr,code_addr,code_filez,code_memsz,5,4096
        dc.l 1,data_offset,data_addr,data_addr,data_filez,data_memsz,6,4096

;--------------------------- code ------------------------------------------

main:   move.l  r7,[stack_ptr] ; save initial stack pointer   

        ; ******************** get socket handle ***************************
        moveq.l 0,-[sp]         ; no protocol specified
        moveq.l 1,-[sp] ; 1: SOCK_STREAM (/usr/include/linux/net.h)
        moveq.l 1,-[sp] ; 1: AF_UNIX, AF_LOCAL  (/usr/include/linux/socket.h)
        move.l  r7,r2           ; pointer to parameter for "socket"
        move.l  1,r3            ; "socket" (/usr/include/linux/net.h)
        move.l  102,r0          ; socketcall (/usr/include/asm/unistd.h)
        trap    $80
        addq.l  3*4,r7          ; free space for parameters
        cmp.l   -4095,r0        ; ERROR
        bhs.l   err
        move.l  r0,[x_handle]

        ; ********** connect socket to /tmp/.X11-unix/X0" ******************
        moveq.l sockaddr_un_l,-[sp]
        move.l  sockaddr_un,-[sp]    ; (/usr/include/linux/un.h)
        move.l  [x_handle],-[sp]      ; socket handle
        move.l  r7,r2           ; pointer to parameter for "connect"
        move.l  3,r3            ; "connect" (/usr/include/linux/net.h)
        move.l  102,r0          ; socketcall (/usr/include/asm/unistd.h)
        trap    $80
        addq.l  3*4,r7          ; free space for parameters
        cmp.l   -4095,r0        ; ERROR
        bhs.l   err

        ; ******************* send connect message *************************
        move.l  send1,r0        ; pointer to connect message
        move.l  send1l,r1

        bsr.l   get_xauth       ; try to read .Xauthority
        bcs.b   .11             ; no success, let's try without auth.

        move.w  r3,[send1+6]    ; insert name length
        move.w  r5,[send1+8]    ; insert data length
        bsr.l   x_send          ; send header

        move.l  r2,r0           ; pointer to name
        lea.l   [r3 +3],r1      ; pad to a multiple of 4
        andq.l  $0fffffffc,r1
        bsr.l   x_send          ; send name
        
        move.l  r4,r0           ; pointer to data
        lea.l   [r5+3],r1       ; pad to a multiple of 4
        andq.l  $0fffffffc,r1
   .11: bsr.l   x_send          ; send data
        
        eor.l   r4,r4           ; number of total bytes read
        move.l  buf2,r5         ; pointer to buffer for next read
        move.l  buf2l,r6        ; max. bytes to read
   .10: move.l  r5,r0
        move.l  r6,r1
        bsr.l   x_receive_raw
        beq.b   .10             ; but we need a reply

        cmp.b   1,[buf2]        ; success
        bne.l   err             ; something went wrong

        add.l   r0,r4           ; total read bytes
        add.l   r0,r5           ; pointer to buffer for next read
        sub.l   r0,r6           ; max. bytes to read
        cmpq.l   8,r4           ; at least 8 bytes read?
        blo.b   .10             ; no, get more
        
        movu.wl [buf2+6],r3     ; additional data in 4 bytes
        lea.l   [r3*4+8],r3     ; total size in bytes
        cmp.l   r3,r4           ; all read
        blo.b   .10             ; no, get more
        

        ; ******************* calculate id's *******************************
        move.l  buf2,r5
        move.l  [r5+$0c],r0     ; resource_id_base

        move.l  [r5+$10],r1     ; resource_id_mask
        move.l  r1,r2
        neg.l   r2
        and.l   r2,r1           ; resource_id_incr

        move.l  r0,[s2a]        ; wid for CreateWindow
        move.l  r0,[s3a]        ; wid for MapWindow
        move.l  r0,[s5a]        ; wid for CreateDC
        move.l  r0,[s8a]        ; wid for PolyArcFill
        move.l  r0,[s9a]        ; wid for ImageText8

        add.l   r1,r0           ; next id
        move.l  r0,[s5b]        ; cid for CreateDC
        move.l  r0,[s6b]        ; cid for ChangeDC
        move.l  r0,[s7b]        ; cid for ChangeDC
        move.l  r0,[s8b]        ; cid for PolyArcFill
        move.l  r0,[s9b]        ; cid for ImageText8

        add.l   r1,r0           ; next id
        move.l  r0,[s4c]        ; fid for OpenFont
        move.l  r0,[s5c]        ; fid for CreatDc
        
        add.l   r1,r0           ; next id
;       move.l  r0,[resource_id_next]   ; maybe we need more id's later
;       move.l  r1,[resource_id_incr]   ; maybe we need more id's later


        ; ******************* get root window id ***************************
        movu.wl [r5+$18],r0     ; length of vendor string
        addq.l  $28+3,r0        ; const header length + round vendor length
        and.b   $0fc,r0         ; round to 4 bytes

        movu.bl [r5+$1d],r1     ; number of FORMATs
        lsl.l   3,r1            ; 8 byte for each FORMAT entry
        add.l   r0,r1           ; offset to root WINDOW id

        move.l  [r5+r1],r0      ; root window
        move.l  r0,[s2b]        ; CreateWindow needs root window id

        ; ******************* send CreatWindow request *********************
        move.l  send2,r0
        move.l  send2l,r1
        bsr.l   x_send

        ; ******************* send MapWindow request ***********************
        move.l  send3,r0
        move.l  send3l,r1
        bsr.l   x_send

        ; ******************* send OpenFont request *************************
        move.l  send4,r0
        move.l  send4l,r1
        bsr.l   x_send

        ; ******************* send CreatDC request *************************
        move.l  send5,r0
        move.l  send5l,r1
        bsr.l   x_send

        ; ******************** main loop ***************************
loop_:  bsr.l   display
   .20: bsr.l   x_receive

        cmp.b   0,[r0.l]        ; error message
        beq.l   err
        cmp.b   2,[r0.l]        ; key press
        bne.b   .30
        cmp.b   $1b,[r0.l+1]    ; r
        bne.b   .50
        move.l  0,[brett]
        move.l  0,[brett+4]
        br.b    loop_
   .50: cmp.b   $09,[r0.l+1]    ; ESC
        bne.b   .20             ; ignore anything else
        br.b    ende

   .30: cmp.b   4,[r0.l]        ; button press
        bne.b   .10

        move.l  [r0+24],r0
        eor.l   r1,r1
        move.l  50,r2
        divu.w  r2,r1|r0
        not.b   r0
        move.l  r0,r3
        andq.l  7,r3            ; x: 0-7
        lsr.l   16,r0
        eor.l   r1,r1
        divu.w  r2,r1|r0
        andq.l  7,r0            ; y: 0-7
        bchg.l  r3,[r0+brett]
        bchg.l  r3,[r0+brett-1]
        bchg.l  r3,[r0+brett+1]
        inc.l   r3
        tst.b   1000b,r3
        bne.b   .40
        bchg.l  r3,[r0+brett]
   .40: subq.l  2,r3
        bmi.l   loop_
        bchg.l  r3,[r0+brett]
        br.l    loop_
   .10: cmp.b   12,[r0.l]       ; Expose
        beq.l   loop_
        br.l    .20             ; ignore anything else
err:        
ende:   move.l  0,r3            ; return code
        move.l  1,r0            ; exit
        trap    $80



display:move.l  brett+8,r6

        lea.l   [r6 -8],r5
        move.w  0,[s8y]
   .10: move.w  0,[s8x]
        move.l  8,r2
        move.b  [r5.l],r3
        inc.l   r5
   .11: lsl.b   1,r3
        bcc.b   .12
        move.l  send8,r0        ; circle
        move.l  send8l,r1
        bsr.l   x_send
   .12: addq.w  50,[s8x]
        dbf.l   r2,.11
        addq.w  50,[s8y]
        cmp.l   r6,r5
        blo.l   .10

        move.l  send7,r0        ; color2
        move.l  send7l,r1
        bsr.l   x_send
        move.l  'asm',[s9t]

        lea.l   [r6-8],r5
        move.w  0,[s8y]
        move.w  30,[s9y]
   .20: move.w  0,[s8x]
        move.w  5,[s9x]
        move.l  8,r2
        move.b  [r5.l],r3
        inc.l   r5
   .21: lsl.b   1,r3
        bcc.b   .22
        move.l  send9,r0        ; text
        move.l  send9l,r1
        bsr.l   x_send
        br.b    .23
   .22: move.l  send8,r0        ; circle
        move.l  send8l,r1
        bsr.l   x_send
   .23: addq.w  50,[s8x]
        addq.w  50,[s9x]
        dbf.l   r2,.21
        addq.w  50,[s8y]
        addq.w  50,[s9y]
        cmp.l   r6,r5
        blo.l   .20

        move.l  send6,r0        ; color1
        move.l  send6l,r1
        bsr.l   x_send
        move.l  'HLA',[s9t]

        lea.l   [r6 -8],r5
        move.w  30,[s9y]
   .30: move.w  5,[s9x]
        move.l  8,r2
        move.b  [r5.l],r3
        inc.l   r5
   .31: lsl.b   1,r3
        bcs.b   .32
        move.l  send9,r0        ; text
        move.l  send9l,r1
        bsr.l   x_send
   .32: addq.w  50,[s9x]
        dbf.l   r2,.31
        addq.w  50,[s9y]
        cmp.l   r6,r5
        blo.l   .30
        rts.l
 

;**********************************************************
;******** read cookie from $home/.Xauthority **************
;**********************************************************
;                                                         *
; input:  stack_ptr: original sp at program start         *
; output: C=0: cookie found in $home/.Xauthority          *
;              r2: pointer to protocol name               *
;              r3: length of protocol name                *
;              r4: pointer to protocol data               *
;              r5: length of protocol data                *
;         C=1: nothing found                              *
;              r2/r3/r4/r5 undefined                      *
;                                                         *
;       typedef struct xauth {                            *
;            unsigned short family;                       *
;            unsigned short address_length;               *
;            char           *address;                     *
;            unsigned short number_length;                *
;            char           *number;                      *
;            unsigned short name_length;                  *
;            char           *name;                        *
;            unsigned short data_length;                  *
;            char           *data;                        *
;       } Xauth;                                          *
;**********************************************************

get_xauth:
        move.l  r0,-[sp]
        move.l  r1,-[sp]
        move.l  r6,-[sp]
        
        move.l  [stack_ptr],r6  ; original stack pointer at program start
        move.l  [r6],r0         ; number of arguments
        lea.l   [r6+r0*4+8],r6  ; skip arguments + trailing null pointer

   .20: move.l  [r6],r5         ; pointer to next env variable
        addq.l  4,r6
        tst.l   r5,r5           ; no more env variables
        beq.l   .notfound
        cmp.l   'HOME',[r5]     ; HOME found?
        bne.b   .20             ; no, try next
        cmp.b   '=',[r5.l+4]    ; HOME= found?
        bne.b   .20             ; no, try next
        addq.l  5,r5            ; start of HOME path
        orq.l   -1,r2
   .30: inc.l   r2              ; count length of HOME path
        cmp.b   0,[r5.l+r2.l]
        bne.b   .30

        or.l    r2,r2           ; at least one char long?
        beq.l   .notfound       ; no, HOME is empty
        cmp.l   256,r2          ; more than 256 charcters
        bhi.b   .notfound       ; somebody tries a buffer overflow
        move.l  fname,r6        ; buffer for filename
        rep_r2  move.b  [r5]+-,[r6]+-{s1} ; copy HOME path
        move.l  '/.Xa',r0       ; add .Xauthority
        move.l  r0,[r6]+-{s1}
        move.l  'utho',r0
        move.l  r0,[r6]+-{s1}
        move.l  'rity',r0
        move.l  r0,[r6]+-{s1}
        move.b  0,[r6.l]        ; and a trailing 0

        move.l  fname,r3
        eor.l   r2,r2           ; readonly
        move.l  5,r0            ; open
        trap    $80

        cmp.l   -4095,r0        ; file open error?
        bhs.b   .notfound       ; yes
        
        move.l  r0,r3           ; file handle
        move.l  buf2,r2
        move.l  buf2l,r1        ; read 1024 byte
        move.l  3,r0            ; read
        trap    $80

        cmp.l   buf2l,r0
        bhs.l   err             ; .Xauthority >= 1024 byte
        move.l  r0,r4           ; bytes read
        
        move.l  6,r0            ; close
        trap    $80

        tst.l   r4,r4           ; file empty
        beq.b   .notfound

        move.l  buf2,r5
        add.l   r5,r4           ; end of read data
        eor.l   r0,r0           ; delete upper 16 bit of r0
        
   .60: move.w  [r5]+-,r0       ; family
        dec.w   r0
        beq.b   .40             ; 1=FamilyLocal
        
        move.l  4,r2            ; skip entry
   .50: move.w  [r5]+-,r0
        ror.w   8,r0            ; big -> little endian
        add.l   r0,r5
        dbf.l   r2,.50
        cmp.l   r4,r5           ; more data
        blo.b   .60             ; try next entry
        
.notfound:
        bset.w  0,sr            ; set Carry
        br.b    .70
        
   .40: move.l  2,r2
        move.l  r2,r3
   .41: move.w  [r5]+-,r0       ; size of data
        ror.w   8,r0            ; big endian <-> little endian
        add.l   r0,r5           ; skip address/number
        dbf.l   r2,.41

   .42: move.w  [r5]+-,r0       ; size of protocol name
        ror.w   8,r0            ; big endian <-> little endian
        move.l  r5,r2           ; pointer to protocol name
        move.l  r0,r3           ; size of protocol name
        add.l   r3,r5           ; skip name
        
        move.w  [r5]+-,r0       ; size of protocol data
        ror.w   8,r0            ; big endian <-> little endian
        move.l  r5,r4           ; pointer to protocol data
        move.l  r0,r5           ; size of protocol data
        bclr.w  0,sr            ; clear carry
   .70: move.l  [sp]+,r6
        move.l  [sp]+,r1
        move.l  [sp]+,r0
        rts.l   

;**********************************************************
;********   send message to X server         **************
;**********************************************************
; input: r0: pointer to message                           *
;        r1: length of message                            *
;**********************************************************
x_send: movem.l r0-r7,-[sp]
        move.l  r0,r4           ; pointer to next byte of message
        move.l  r1,r5           ; remaining bytes to send

   .20: moveq.l 0,-[sp]         ; flags
        move.l  r5,-[sp]        ; length
        move.l  r4,-[sp]        ; pointer to data
        move.l  [x_handle],-[sp]; socket handle
        move.l  r7,r2           ; pointer to parameter for "send"
        move.l  9,r3            ; "send"  (/usr/include/linux/net.h)
        move.l  102,r0          ; socketcall (/usr/include/asm/unistd.h)
        trap    $80
        addq.l  4*4,r7          ; free space for parameters

        cmpq.l  -11,r0          ; EAGAIN:
        beq.b   .20             ; message couldn't be sent, try again

        cmp.l   -4095,r0        ; ERROR
        bhs.l   err

        sub.l   r0,r5           ; remaining bytes to send
        beq.b   .30             ; nothing, all sent
        add.l   r0,r4           ; pointer to remaining message
        br.b    .20             ; send rest of message

   .30: movem.l [sp]+,r0-r7
        rts.l

;**********************************************************
;********   receive ONE message from X server    **********
;**********************************************************
; input: none                                             *
; output: Z=1: no complete message available              *
;              r0/r1 undefined                            *
;         Z=0: r0: pointer to message data                *
;              r1: size of data                           *
;**********************************************************
x_receive:
        move.l  r2,-[sp]
        move.l  r5,-[sp]
        move.l  r6,-[sp]
   .00: move.l  [buf2_rest],r0  ; still something in read buffer?
        cmpq.l  32,r0           ; a message has at least 32 bytes
        bhs.b   .10             ; maybe it is a complete message

   .30: move.l  [buf2_ptr],r5   ; start of message
        move.l  buf2,r6         ; start of buffer
        move.l  r6,[buf2_ptr]   ; we copy message to top of buffer 
        cmp.l   r5,r6           ; already at top of buffer
        beq.b   .50             ; then nothing to copy
        or.l    r0,r0           ; nothing in buffer
        beq.b   .50             ; then also nothing to copy
        move.l  r0,r2           ; copy to top of buffer
        rep_r2  move.b  [r5]+-,[r6]+-{s1}
        
   .50: move.l  buf2l,r1        ; let's try to get some more data
        sub.l   r0,r1           ; not more bytes than space is left in the buf
        lea.l   [r0+buf2],r0    ; append it here
        bsr.l   x_receive_raw
        bne.b   .20             ; we could read something
        br.b    .100            ; return with Z=1

   .20: add.l   r0,[buf2_rest]  ; now we have a few more bytes in the buffer
        br.b    .00             ; let's try again
        
   .10: move.l  [buf2_ptr],r5   ; let's test if it is a complete meesage
        move.l  32,r1           ; error/reply/event base length
        cmp.b   1,[r5.l]        ; reply message
        bne.b   .40             ; no, then event/error messages (always 32 byte)
        add.l   [r5+4],r1       ; + additional data for reply
        add.l   [r5+4],r1       ; + additional data for reply
        add.l   [r5+4],r1       ; + additional data for reply
        add.l   [r5+4],r1       ; + additional data for reply
        cmp.l   r1,r0           ; complete reply in buffer
        blo.b   .30             ; no, let's try to get more
   .40: move.l  r5,r0           ; pointer to data
        sub.l   r1,[buf2_rest]  ; new rest
        add.l   r1,[buf2_ptr]   ; pointer to next data; clear Z flag
  .100: move.l  [sp]+,r6
        move.l  [sp]+,r5
        move.l  [sp]+,r2
        rts.l

;**********************************************************
;********      read data from X server           **********
;**********************************************************
; input: r0: pointer to read buffer                       *
;        r1: size of buffer                               *
; output: Z=1: nothing to read                            *
;         Z=0: r0 bytes read                              *
;**********************************************************
x_receive_raw:
        move.l  r2,-[sp]
        move.l  r3,-[sp]

        moveq.l 0,-[sp]         ; flags
        move.l  r1,-[sp]        ; number of bytes to read 
        move.l  r0,-[sp]        ; pointer to buffer
        move.l  [x_handle],-[sp]; socket handle
        move.l  r7,r2           ; pointer to parameter for "recv"
        move.l  10,r3           ; "recv"  (/usr/include/linux/net.h)
        move.l  102,r0          ; socketcall (/usr/include/asm/unistd.h)
        trap    $80
        addq.l  4*4,r7          ; free space for parameters
        move.l  [sp]+,r3
        move.l  [sp]+,r2
        cmpq.l  -11,r0          ; EAGAIN: no message available -> Z=1
        beq.b   .10

        cmp.l   -4095,r0           ; ERROR
        bhs.l   err

        tst.l   r0,r0           ;  0: NULL message -> Window close?
        beq.l   ende

   .10: rts.l

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;--------------------------- constant data ---------------------------------
        align   4

sockaddr_un:    
        dc.w    1     ; 1: AF_UNIX, AF_LOCAL  (/usr/include/linux/socket.h)
        dc.b    "/tmp/.X11-unix/X0"
                                        sockaddr_un_l equ $-sockaddr_un
        align   4

;---------------------------------------------------------------------------

        align 4
        code_memsz equ $-$$
        code_filez equ code_memsz
        data_addr equ (orig+code_memsz+4095)/4096*4096 + (code_filez % 4096)
        data_offset equ code_filez
        section .data vstart=data_addr

;--------------------------- initialized data ------------------------------
buf2_ptr:       dc.l    buf2
buf2_rest:      dc.l    0

        ; Connection Setup   
send1:  dc.b    $6c,0   ; LSB first
        dc.w    11,0    ; major/minor version
        dc.w    0,0     ; length of protocol name/data
        dc.w    0       ; unused
                          send1l equ $-send1

        ; Create Window
send2:  dc.b    1       ; opcode for Create Window
        dc.b    0       ; depth from parent
        dc.w    send2l/4; request length
   s2a: dc.l    0       ; wid (has to be calculated)
   s2b: dc.l    0       ; parent (has to be calculated)
        dc.w    0       ; x
        dc.w    0       ; y
   s2x: dc.w    400     ; with
   s2y: dc.w    400     ; higth
        dc.w    0       ; border-width
        dc.w    0       ; class: CopyFromParent
        dc.l    0       ; visual: CopyFromParent
        dc.l    $0802   ; value-mask:   background-pixel    2 
                        ;             + event-mask        800
        dc.l    $08080ff; background: light blue
        dc.l    $08005  ; event_mask: KeyPress             1
                        ;            +ButtenPress          4
                        ;            +PointerMotion      $40
                        ;            +Exposure         $8000
                        ;            +StructureNotify $20000
                          send2l equ $-send2
                          
        ; Map Window
send3:  dc.b    8       ; opcode for Map Window
        dc.b    0       ; unused
        dc.w    send3l/4; request length
   s3a: dc.l    0       ; wid (has to be calculated)
                          send3l equ $-send3

        ; Open Font
send4:  dc.b    45      ; opcode for OpenFont
        dc.b    0       ; unused
        dc.w    send4l/4; request length
  s4c:  dc.l    0       ; fid (has to be calculated)
        dc.w    s4tl
        dc.w    0       ; unused
  s4t:  dc.b    "lucidasans-bold-12"    ; font name
        s4tl equ $-s4t
        align 4
                          send4l equ $-send4

        ; Create GC
send5:  dc.b    55      ; opcode for CreateGC
        dc.b    0       ; unused
        dc.w    send5l/4; request length
   s5b: dc.l    0       ; cid (has to be calculated)
   s5a: dc.l    0       ; wid (has to be calculated)
        dc.l    1+4+8+$4000  ; function+foreground+background+font
        dc.l    3       ; function=copy
        dc.l    $0e02020; foreground: red
        dc.l    $020e020; background: green
   s5c: dc.l    0       ; fid (hast to be calculated)
                          send5l equ $-send5

        ; Change GC
send6:  dc.b    56      ; opcode for ChangeGC
        dc.b    0       ; unused
        dc.w    send6l/4; request length
   s6b: dc.l    0       ; cid (has to be calculated)
        dc.l    4+8     ; foreground+background
        dc.l    $020e020; foreground: green
        dc.l    $0e02020; background: red
                          send6l equ $-send6
        ; Change GC
send7:  dc.b    56      ; opcode for ChangeGC
        dc.b    0       ; unused
        dc.w    send7l/4; request length
   s7b: dc.l    0       ; cid (has to be calculated)
        dc.l    4+8     ; foreground+background
        dc.l    $0e02020; foreground: red
        dc.l    $020e020; background: green
                          send7l equ $-send7

send8:  dc.b    71      ; opcode for PolyFillArc
        dc.b    0       ; unused
        dc.w    send8l/4; request length
   s8a: dc.l    0       ; wid (has to be calculated)
   s8b: dc.l    0       ; cid (has to be calculated)
   s8x: dc.w    0       ; x
   s8y: dc.w    0       ; y
        dc.w    50      ; w
        dc.w    50      ; h
        dc.w    0*64    ; start angle *64       
        dc.w    360*64  ; end angle *64 
                          send8l equ $-send8


send9:  dc.b    76      ; opcode for ImageText8
        dc.b    s9tl    ; string length
        dc.w    send9l/4; request length
   s9a: dc.l    0       ; wid (has to be calculated)
   s9b: dc.l    0       ; cid (has to be calculated)
   s9x: dc.w    0       ; x
   s9y: dc.w    30      ; y
   s9t: dc.b    "HLA"   ; string
        s9tl equ $-s9t
        align 4
                          send9l equ $-send9


        dc.b    0       ; we need this byte
brett:  dc.b    0,0,0,0,0,0,0,0
        dc.b    0       ; we need this byte
                          
;---------------------------------------------------------------------------

        idat_memsz equ $-$$
        bss_addr equ data_addr+ ($-$$)
        section .bss vstart=bss_addr

;--------------------------- uninitialized data ----------------------------

stack_ptr:      blk.l   1
x_handle:       blk.l   1
fname:          blk.b   256+32
buf2:           blk.b   16*1024
                buf2l equ $-buf2

;---------------------------------------------------------------------------
 
        udat_memsz equ $-$$
        data_memsz equ idat_memsz + udat_memsz
        data_filez equ idat_memsz
;===========================================================================





